@@ -1,9 +1,3 @@
-# -*- coding: utf-8 -*-
-# django false-positives:
-# pylint: disable=unexpected-keyword-arg,no-value-for-parameter
-from __future__ import unicode_literals
-from __future__ import absolute_import
-
 import os
 import copy
 import contextlib
@@ -12,13 +6,13 @@ import glob
 import re
 import logging
 
-import debian.debian_support
-
 import django.db
 import django.core.exceptions
 import django.contrib.auth.models
 
-import mini_buildd.setup
+import debian.debian_support
+
+import mini_buildd.config
 import mini_buildd.misc
 import mini_buildd.gnupg
 import mini_buildd.reprepro
@@ -40,7 +34,7 @@ class EmailAddress(mini_buildd.models.base.Model):
     class Admin(mini_buildd.models.base.Model.Admin):
         exclude = ("extra_options",)
 
-    def __unicode__(self):
+    def __str__(self):
         return "{n} <{a}>".format(n=self.name, a=self.address)
 
 
@@ -52,12 +46,13 @@ class Suite(mini_buildd.models.base.Model):
     class Admin(mini_buildd.models.base.Model.Admin):
         exclude = ("extra_options",)
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
-    def clean(self, *args, **kwargs):
+    # Note: pylint false-positive: https://github.com/PyCQA/pylint/issues/1553
+    def clean(self, *args, **kwargs):  # pylint: disable=arguments-differ
         self.mbd_validate_regex(r"^[a-z]+$", self.name, "Name")
-        super(Suite, self).clean(*args, **kwargs)
+        super().clean(*args, **kwargs)
 
 
 class SuiteOption(mini_buildd.models.base.Model):
@@ -102,15 +97,16 @@ lintian) as non-lethal, and will install anyway.
     class Meta(mini_buildd.models.base.Model.Meta):
         unique_together = ("suite", "layout")
 
-    def __unicode__(self):
-        return "{l}: {e}{n}{e} [{u}]{m}".format(
-            l=self.layout.name,
+    def __str__(self):
+        return "{layout}: {e}{n}{e} [{u}]{m}".format(
+            layout=self.layout.name,
             n=self.suite.name,
             e="*" if self.experimental else "",
             u="uploadable" if self.uploadable else "managed",
             m=" => {m}".format(m=self.migrates_to.suite.name) if self.migrates_to else "")
 
-    def clean(self, *args, **kwargs):
+    # Note: pylint false-positive: https://github.com/PyCQA/pylint/issues/1553
+    def clean(self, *args, **kwargs):  # pylint: disable=arguments-differ
         if self.build_keyring_package and not self.uploadable:
             raise django.core.exceptions.ValidationError("You can only build keyring packages on uploadable suites!")
         if self.experimental and self.migrates_to:
@@ -122,11 +118,11 @@ lintian) as non-lethal, and will install anyway.
         if self.migrates_to and self.migrates_to.uploadable:
             raise django.core.exceptions.ValidationError("You may not migrate to an uploadable suite.")
 
-        super(SuiteOption, self).clean(*args, **kwargs)
+        super().clean(*args, **kwargs)
 
     @property
     def rollback(self):
-        " Rollback field temporarily implemented as extra_option. "
+        """Rollback field temporarily implemented as extra_option."""
         return int(self.mbd_get_extra_option("Rollback", "0"))
 
     def mbd_get_distribution_string(self, repository, distribution, rollback=None):
@@ -137,10 +133,10 @@ lintian) as non-lethal, and will install anyway.
 
         if rollback is None:
             return dist_string
-        else:
-            if rollback not in range(self.rollback):
-                raise Exception("Rollback number out of range: {r} ({m})".format(r=rollback, m=self.rollback))
-            return "{d}-rollback{r}".format(d=dist_string, r=rollback)
+
+        if rollback not in list(range(self.rollback)):
+            raise Exception("Rollback number out of range: {r} ({m})".format(r=rollback, m=self.rollback))
+        return "{d}-rollback{r}".format(d=dist_string, r=rollback)
 
     def mbd_get_apt_pin(self, repository, distribution):
         return "release n={c}, o={o}".format(
@@ -153,9 +149,7 @@ lintian) as non-lethal, and will install anyway.
             prio=prio)
 
     def mbd_get_sort_no(self):
-        """
-        Compute number that may be used to sort suites from 'stable' (0) towards 'experimental'.
-        """
+        """Compute number that may be used to sort suites from 'stable' (0) towards 'experimental'."""
         no = 0
         if self.uploadable:
             no += 5
@@ -224,7 +218,7 @@ packages (to unstable,experimental,..) aimed for Debian.
 
         @classmethod
         def mbd_meta_create_defaults(cls, msglog):
-            "Create default layouts and suites."
+            """Create default layouts and suites."""
             stable, created = Suite.mbd_get_or_create(msglog, name="stable")
             hotfix, created = Suite.mbd_get_or_create(msglog, name="hotfix")
             testing, created = Suite.mbd_get_or_create(msglog, name="testing")
@@ -232,14 +226,13 @@ packages (to unstable,experimental,..) aimed for Debian.
             snapshot, created = Suite.mbd_get_or_create(msglog, name="snapshot")
             experimental, created = Suite.mbd_get_or_create(msglog, name="experimental")
 
-            for name, extra_options in {"Default": {"stable": "Rollback: 6\n",
-                                                    "hotfix": "Rollback: 4\n",
-                                                    "testing": "Rollback: 3\n",
-                                                    "unstable": "Rollback: 9\n",
-                                                    "snapshot": "Rollback: 12\n",
-                                                    "experimental": "Rollback: 6\n"},
-                                        "Default (no rollbacks)": {}}.items():
-
+            for name, extra_options in list({"Default": {"stable": "Rollback: 6\n",
+                                                         "hotfix": "Rollback: 4\n",
+                                                         "testing": "Rollback: 3\n",
+                                                         "unstable": "Rollback: 9\n",
+                                                         "snapshot": "Rollback: 12\n",
+                                                         "experimental": "Rollback: 6\n"},
+                                             "Default (no rollbacks)": {}}.items()):
                 default_layout, created = Layout.mbd_get_or_create(msglog, name=name)
                 if created:
                     so_stable = SuiteOption(
@@ -310,7 +303,7 @@ packages (to unstable,experimental,..) aimed for Debian.
                     but_automatic_upgrades=False)
                 debdev_experimental.save()
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
     @classmethod
@@ -331,7 +324,7 @@ packages (to unstable,experimental,..) aimed for Debian.
             repository, distribution)
 
     def mbd_get_reverse_dependencies(self):
-        "When the layout changes, all repos that use that layout also change."
+        """When the layout changes, all repos that use that layout also change."""
         return [r for r in self.repository_set.all()]
 
 
@@ -352,13 +345,14 @@ class ArchitectureOption(mini_buildd.models.base.Model):
     class Meta(mini_buildd.models.base.Model.Meta):
         unique_together = ("architecture", "distribution")
 
-    def __unicode__(self):
+    def __str__(self):
         return "{a} for {d}".format(a=self.architecture, d=self.distribution)
 
-    def clean(self, *args, **kwargs):
+    # Note: pylint false-positive: https://github.com/PyCQA/pylint/issues/1553
+    def clean(self, *args, **kwargs):  # pylint: disable=arguments-differ
         if self.build_architecture_all and self.optional:
             raise django.core.exceptions.ValidationError("Optional architectures must not be architecture all!")
-        super(ArchitectureOption, self).clean(*args, **kwargs)
+        super().clean(*args, **kwargs)
 
 
 class ArchitectureOptionInline(django.contrib.admin.TabularInline):
@@ -496,7 +490,7 @@ $build_environment = { 'CCACHE_DIR' => '%LIBDIR%/.ccache' };
                                                   help_text="""\
 Perl snippet to be added in the .sbuildrc for each build.; you may use these placeholders:<br />
 <br />
-%LIBDIR%: Per-chroot persistent dir; may be used for data that should persist beteeen builds (caches, ...).<br />
+%LIBDIR%: Per-chroot persistent dir; may be used for data that should persist between builds (caches, ...).<br />
 <br />
 Example:
 <pre>
@@ -565,7 +559,7 @@ some reason create their automated debug packages with file appendix
 
         @classmethod
         def mbd_meta_add_base_sources(cls, msglog):
-            "Add default distribution objects for all base sources found."
+            """Add default distribution objects for all base sources found."""
             def default_components(origin):
                 return {"Ubuntu": ["main", "universe", "restricted", "multiverse"]}.get(origin, ["main", "contrib", "non-free"])
 
@@ -601,7 +595,7 @@ some reason create their automated debug packages with file appendix
                     # Save changes to Distribution model
                     new_dist.save()
 
-    def __unicode__(self):
+    def __str__(self):
         return "{o} '{b}': {c} ({a}) + ({x})".format(o=self.base_source.origin,
                                                      b=self.base_source.codename,
                                                      c=" ".join(self.mbd_get_components()),
@@ -609,7 +603,7 @@ some reason create their automated debug packages with file appendix
                                                      x=", ".join(["{c}:{p}".format(c=e.source.codename, p=e.priority) for e in self.extra_sources.all()]))
 
     def mbd_get_components(self):
-        return [c.name for c in sorted(self.components.all(), cmp=mini_buildd.models.source.cmp_components)]
+        return [c.name for c in sorted(self.components.all(), key=mini_buildd.models.source.component_key)]
 
     def mbd_get_architectures(self, show_opt_flag=False):
         return ["{a}{o}".format(a=a.architecture.name, o="*" if (show_opt_flag and a.optional) else "") for a in self.architectureoption_set.all()]
@@ -624,7 +618,7 @@ some reason create their automated debug packages with file appendix
         return "{p}{u}{r}/{i}/ {d} {c}".format(
             p=prefix,
             u=self.mbd_get_daemon().model.mbd_get_http_url(),
-            r=os.path.basename(mini_buildd.setup.REPOSITORIES_DIR),
+            r=os.path.basename(mini_buildd.config.REPOSITORIES_DIR),
             i=repository.identity,
             d=suite_option.mbd_get_distribution_string(repository, self, rollback=rollback),
             c=" ".join(self.mbd_get_components()))
@@ -663,7 +657,7 @@ some reason create their automated debug packages with file appendix
         return mini_buildd.misc.fromdos(mini_buildd.misc.subst_placeholders(self.sbuildrc_snippet, {"LIBDIR": mini_buildd.misc.chroot_libdir_path(self.base_source.codename, arch)}))
 
     def mbd_get_reverse_dependencies(self):
-        "When the distribution changes, all repos that use that distribution also change."
+        """When the distribution changes, all repos that use that distribution also change."""
         return [r for r in self.repository_set.all()]
 
 
@@ -721,45 +715,31 @@ Example:
         filter_horizontal = ("distributions", "notify",)
 
         def get_readonly_fields(self, _request, obj=None):
-            "Forbid change identity on existing repository."
+            """Forbid change identity on existing repository."""
             fields = copy.copy(self.readonly_fields)
             if obj:
                 fields.append("identity")
             return fields
 
         @classmethod
-        def mbd_meta_build_keyring_packages(cls, msglog):
-            if not Repository.mbd_get_daemon().is_running():
-                raise Exception("Daemon needs to be running to build keyring packages")
-            for s in Repository.mbd_get_active():
-                s.mbd_build_keyring_packages(msglog.request)
-
-        @classmethod
-        def mbd_meta_build_test_packages(cls, msglog):
-            if not Repository.mbd_get_daemon().is_running():
-                raise Exception("Daemon needs to be running to build test packages")
-            for s in Repository.mbd_get_active():
-                s.mbd_build_test_packages(msglog.request)
-
-        @classmethod
-        def mbd_meta_add_sandbox(cls, msglog):
-            "Add sandbox repository 'test'."
-            sandbox_repo, created = Repository.mbd_get_or_create(
+        def mbd_meta_add_test(cls, msglog):
+            """Add sandbox repository 'test'."""
+            test_repo, created = Repository.mbd_get_or_create(
                 msglog,
                 identity="test",
                 allow_unauthenticated_uploads=True,
                 layout=Layout.objects.get(name__exact="Default"))
             if created:
                 for d in Distribution.objects.all():
-                    sandbox_repo.distributions.add(d)
+                    test_repo.distributions.add(d)
 
         @classmethod
         def mbd_meta_add_debdev(cls, msglog):
-            "Add developer repository 'debdev', only for sid."
+            """Add developer repository 'debdev', only for sid."""
             try:
                 sid = Distribution.objects.get(base_source__codename="sid")
-            except:
-                raise Exception("No 'sid' distribution found")
+            except BaseException as e:
+                raise Exception("No 'sid' distribution found: {e}".format(e=e))
             debdev_repo, created = Repository.mbd_get_or_create(
                 msglog,
                 identity="debdev",
@@ -771,32 +751,13 @@ Example:
             if created:
                 debdev_repo.distributions.add(sid)
 
-    def __unicode__(self):
+    def __str__(self):
         return "{i}: {d}".format(i=self.identity, d=" ".join([d.base_source.codename for d in self.distributions.all()]))
 
-    def clean(self, *args, **kwargs):
+    # Note: pylint false-positive: https://github.com/PyCQA/pylint/issues/1553
+    def clean(self, *args, **kwargs):  # pylint: disable=arguments-differ
         self.mbd_validate_regex(r"^[a-z0-9]+$", self.identity, "Identity")
-        super(Repository, self).clean(*args, **kwargs)
-
-    def _mbd_portext2keyring_suites(self, request, dsc_url):
-        for d in self.distributions.all():
-            for s in self.layout.suiteoption_set.all().filter(build_keyring_package=True):
-                dist = s.mbd_get_distribution_string(self, d)
-                info = "Port for {d}: {p}".format(d=dist, p=os.path.basename(dsc_url))
-                try:
-                    self.mbd_get_daemon().portext(dsc_url, dist)
-                    MsgLog(LOG, request).info("Requested: {i}".format(i=info))
-                except Exception as e:
-                    mini_buildd.setup.log_exception(MsgLog(LOG, request), "FAILED: {i}".format(i=info), e)
-
-    def mbd_build_keyring_packages(self, request):
-        with contextlib.closing(self.mbd_get_daemon().get_keyring_package()) as package:
-            self._mbd_portext2keyring_suites(request, "file://" + package.dsc)
-
-    def mbd_build_test_packages(self, request):
-        for t in ["archall", "cpp", "ftbfs"]:
-            with contextlib.closing(self.mbd_get_daemon().get_test_package(t)) as package:
-                self._mbd_portext2keyring_suites(request, "file://" + package.dsc)
+        super().clean(*args, **kwargs)
 
     def mbd_get_uploader_keyring(self):
         gpg = mini_buildd.gnupg.TmpGnuPG()
@@ -807,23 +768,23 @@ Example:
             uploader = None
             try:
                 uploader = u.uploader
-            except:
-                LOG.warn("User '{u}' does not have an uploader profile (deliberately removed?)".format(u=u))
+            except BaseException as e:
+                LOG.warning("User '{u}' does not have an uploader profile (deliberately removed?): {e}".format(u=u, e=e))
 
             if uploader and uploader.mbd_is_active() and uploader.may_upload_to.all().filter(identity=self.identity):
                 LOG.info("Adding uploader key for '{r}': {k}: {n}".format(r=self, k=uploader.key_long_id, n=uploader.key_name))
                 gpg.add_pub_key(uploader.key)
 
         # Add configured extra keyrings
-        for l in self.extra_uploader_keyrings.splitlines():
-            l = l.strip()
-            if l and l[0] != "#":
-                LOG.info("Adding keyring: {k}".format(k=l))
-                gpg.add_keyring(l)
+        for line in self.extra_uploader_keyrings.splitlines():
+            line = line.strip()
+            if line and line[0] != "#":
+                LOG.info("Adding keyring: {k}".format(k=line))
+                gpg.add_keyring(line)
         return gpg
 
     def mbd_get_path(self):
-        return os.path.join(mini_buildd.setup.REPOSITORIES_DIR, self.identity)
+        return os.path.join(mini_buildd.config.REPOSITORIES_DIR, self.identity)
 
     def mbd_get_description(self, distribution, suite_option):
         return "{s} packages for {d}-{i}".format(s=suite_option.suite.name, d=distribution.base_source.codename, i=self.identity)
@@ -839,10 +800,10 @@ Example:
             return result
         except Exception as e:
             # Raise Exception with human-readable text
-            raise Exception("Please fix syntax error in extra option 'Meta-Distributions' in layout '{l}': {e}".format(l=self.layout, e=e))
+            raise Exception("Please fix syntax error in extra option 'Meta-Distributions' in layout '{layout}': {e}".format(layout=self.layout, e=e))
 
     def mbd_distribution_strings(self, **suiteoption_filter):
-        "Return a list with all full distributions strings, optionally matching a suite options filter (unstable, experimental,...)."
+        """Return a list with all full distributions strings, optionally matching a suite options filter (unstable, experimental,...)."""
         result = []
         for d in self.distributions.all():
             result += [s.mbd_get_distribution_string(self, d) for s in self.layout.suiteoption_set.filter(**suiteoption_filter)]
@@ -869,7 +830,7 @@ Example:
     def mbd_get_internal_suite_dependencies(self, suite_option):
         result = []
 
-        # Add ourselfs
+        # Add ourselves
         result.append(suite_option)
 
         if suite_option.experimental:
@@ -910,7 +871,7 @@ DscIndices: Sources Release . .gz .bz2
                     meta_distributions=" ".join(self.mbd_get_meta_distributions(d, s)),
                     origin=self.mbd_get_daemon().model.mbd_get_archive_origin(),
                     components=" ".join(d.mbd_get_components()),
-                    architectures=" ".join([x.name for x in d.architectures.all()]),  # pylint: disable=no-member
+                    architectures=" ".join([x.name for x in d.architectures.all()]),
                     desc=self.mbd_get_description(d, s),
                     na="yes" if s.not_automatic else "no",
                     bau="yes" if s.but_automatic_upgrades else "no")
@@ -921,7 +882,7 @@ DscIndices: Sources Release . .gz .bz2
                         meta_distributions="",
                         origin=self.mbd_get_daemon().model.mbd_get_archive_origin(),
                         components=" ".join(d.mbd_get_components()),
-                        architectures=" ".join([x.name for x in d.architectures.all()]),  # pylint: disable=no-member
+                        architectures=" ".join([x.name for x in d.architectures.all()]),
                         desc="{d}: Automatic rollback distribution #{r}".format(d=self.mbd_get_description(d, s), r=r),
                         na="yes",
                         bau="no")
@@ -936,30 +897,30 @@ DscIndices: Sources Release . .gz .bz2
         for d in self.distributions.all():
             for s in self.layout.suiteoption_set.all():
                 rollbacks = s.rollback if with_rollbacks else 0
-                for rollback in [None] + range(rollbacks):
+                for rollback in [None] + list(range(rollbacks)):
                     dist_str = s.mbd_get_distribution_string(self, d, rollback)
                     if re.search(dist_regex, dist_str):
                         result.extend(self._mbd_reprepro().list(pattern, dist_str, typ=typ))
         return result
 
-    def mbd_get_dsc_url(self, distribution, package, version):
-        """
-        Get complete DSC URL of an installed package.
-        """
+    def mbd_get_dsc_path(self, distribution, package, version):
+        """Get component and (absolute) DSC path of an installed package (http://host:port/<path>)."""
         subdir = package[:4] if package.startswith("lib") else package[0]
 
-        for c in sorted(distribution.components.all(), cmp=mini_buildd.models.source.cmp_components):
+        for c in sorted(distribution.components.all(), key=mini_buildd.models.source.component_key):
             dsc = "{r}/pool/{c}/{d}/{p}/{dsc}".format(r=self.identity, c=c.name, d=subdir, p=package,
                                                       dsc=mini_buildd.changes.Changes.gen_dsc_file_name(package, version))
             LOG.debug("Checking dsc: {d}".format(d=dsc))
-            if os.path.exists(os.path.join(mini_buildd.setup.REPOSITORIES_DIR, dsc)):
-                return c.name, os.path.join(self.mbd_get_daemon().model.mbd_get_http_url(), os.path.basename(mini_buildd.setup.REPOSITORIES_DIR), dsc)
+            if os.path.exists(os.path.join(mini_buildd.config.REPOSITORIES_DIR, dsc)):
+                return c.name, os.path.join("/", os.path.basename(mini_buildd.config.REPOSITORIES_DIR), dsc)
 
         # Not found in pool
         return None, None
 
     def mbd_package_show(self, package):
         """
+        Show package information.
+
         Result is of the form:
 
         [(CODENAME, [{"distribution": DIST,
@@ -1003,7 +964,7 @@ DscIndices: Sources Release . .gz .bz2
             codename = get_or_create_codename(dist.codename)
 
             # Get component and URL early
-            component, dsc_url = self.mbd_get_dsc_url(distribution, r["source"], r["sourceversion"])
+            component, dsc_path = self.mbd_get_dsc_path(distribution, r["source"], r["sourceversion"])
 
             # Get dict with distribution values
             values = get_or_create_distribution(codename, dist.get(rollback=False), component)
@@ -1013,7 +974,7 @@ DscIndices: Sources Release . .gz .bz2
             values.setdefault("component", component)
             values.setdefault("source", "")
             values.setdefault("sourceversion", "")
-            values.setdefault("dsc_url", "")
+            values.setdefault("dsc_path", "")
             values.setdefault("migrates_to", suite.migrates_to.mbd_get_distribution_string(self, distribution) if suite.migrates_to else "")
             values.setdefault("uploadable", suite.uploadable)
             values.setdefault("experimental", suite.experimental)
@@ -1025,15 +986,15 @@ DscIndices: Sources Release . .gz .bz2
                 # Add to rollback list with "no" appended
                 r["no"] = dist.rollback_no
                 r["component"] = component
-                r["dsc_url"] = dsc_url
+                r["dsc_path"] = dsc_path
                 values["rollbacks"].append(r)
             else:
                 # Copy all reprepro values
-                for k, v in r.items():
+                for k, v in list(r.items()):
                     values[k] = v
 
                 # Extra: URL, is_migrated flag
-                values["dsc_url"] = dsc_url
+                values["dsc_path"] = dsc_path
                 values["is_migrated"] = \
                     values["migrates_to"] and \
                     self._mbd_package_find(pkg_show, distribution=values["migrates_to"], version=values["sourceversion"])
@@ -1041,7 +1002,7 @@ DscIndices: Sources Release . .gz .bz2
         return result
 
     @classmethod
-    def _mbd_package_find(cls, pkg_show, distribution=None, version=None):
+    def _mbd_package_find(cls, pkg_show, distribution=None, version=None):  # pylint: disable=inconsistent-return-statements
         for r in pkg_show:
             if not distribution or r["distribution"] == distribution:
                 if not version or r["sourceversion"] == version:
@@ -1087,11 +1048,11 @@ DscIndices: Sources Release . .gz .bz2
             LOG.info("Rollback: Moving {p}: {s} to {d}".format(p=package_name, s=src, d=dst))
             try:
                 reprepro_output += self._mbd_reprepro().migrate(package_name, src, dst)
-            except Exception as e:
-                mini_buildd.setup.log_exception(LOG, "Rollback failed (ignoring)", e)
+            except BaseException as e:
+                mini_buildd.config.log_exception(LOG, "Rollback failed (ignoring)", e)
         return reprepro_output
 
-    def mbd_package_migrate(self, package, distribution, suite, rollback=None, version=None, msglog=LOG):
+    def _mbd_package_migrate(self, package, distribution, suite, rollback=None, version=None, msglog=LOG):
         reprepro_output = ""
 
         src_dist = suite.mbd_get_distribution_string(self, distribution)
@@ -1142,6 +1103,17 @@ DscIndices: Sources Release . .gz .bz2
 
         return reprepro_output
 
+    def mbd_package_migrate(self, package, distribution, suite, full=False, rollback=None, version=None, msglog=LOG):
+        reprepro_output = ""
+        if full:
+            while suite.migrates_to is not None:
+                reprepro_output += self._mbd_package_migrate(package, distribution, suite, rollback=rollback, version=version, msglog=msglog)
+                suite = suite.migrates_to
+        else:
+            reprepro_output = self._mbd_package_migrate(package, distribution, suite, rollback=rollback, version=version, msglog=msglog)
+
+        return reprepro_output
+
     def mbd_package_remove(self, package, distribution, suite, rollback=None, version=None, msglog=LOG):
         reprepro_output = ""
 
@@ -1166,11 +1138,11 @@ DscIndices: Sources Release . .gz .bz2
                 try:
                     reprepro_output += self._mbd_reprepro().migrate(package, src, dst)
                     reprepro_output += self._mbd_reprepro().remove(package, src)
-                except Exception as e:
-                    mini_buildd.setup.log_exception(msglog,
-                                                    "Rollback: Moving '{p}' from '{s}' to '{d}' FAILED (ignoring)".format(p=package, s=src, d=dst),
-                                                    e,
-                                                    logging.WARN)
+                except BaseException as e:
+                    mini_buildd.config.log_exception(msglog,
+                                                     "Rollback: Moving '{p}' from '{s}' to '{d}' FAILED (ignoring)".format(p=package, s=src, d=dst),
+                                                     e,
+                                                     logging.WARN)
 
         # Finally, purge any now-maybe-orphaned package logs
         self.mbd_package_purge_orphaned_logs(package, msglog=msglog)
@@ -1213,9 +1185,7 @@ DscIndices: Sources Release . .gz .bz2
             LOG.info("Installed: {p} ({d})".format(p=bres.get_pkg_id(with_arch=True), d=dist_str))
 
     def mbd_package_install(self, distribution, suite_option, changes, bresults):
-        """
-        Install a dict arch:bres of successful build results.
-        """
+        """Install a dict arch:bres of successful build results."""
         # Get the full distribution str
         dist_str = suite_option.mbd_get_distribution_string(self, distribution)
 
@@ -1236,7 +1206,7 @@ DscIndices: Sources Release . .gz .bz2
         self._mbd_reprepro().install_dsc(changes.dsc_file_name, dist_str)
 
         # Second, install all build results
-        for bres in bresults.values():
+        for bres in list(bresults.values()):
             # Don't try install if skipped
             if bres.get("Sbuild-Status") == "skipped":
                 LOG.info("Skipped: {p} ({d})".format(p=bres.get_pkg_id(with_arch=True), d=bres["Distribution"]))
@@ -1247,9 +1217,7 @@ DscIndices: Sources Release . .gz .bz2
         self.mbd_package_purge_orphaned_logs(package)
 
     def mbd_prepare(self, _request):
-        """
-        Idempotent repository preparation. This may be used as-is as mbd_sync.
-        """
+        """Idempotent repository preparation. This may be used as-is as mbd_sync."""
         # Architecture sanity checks
         for d in self.distributions.all():
             if not d.architectureoption_set.all().filter(optional=False):
@@ -1269,7 +1237,7 @@ DscIndices: Sources Release . .gz .bz2
                 raise django.core.exceptions.ValidationError("Mandatory component 'main' missing in: {d}".format(d=d))
 
         # (Re-)build config files
-        mini_buildd.misc.mkdirs(os.path.join(self.mbd_get_path(), "conf"))
+        os.makedirs(os.path.join(self.mbd_get_path(), "conf"), exist_ok=True)
         mini_buildd.misc.ConfFile(
             os.path.join(self.mbd_get_path(), "conf", "distributions"),
             self._mbd_reprepro_config()).save()
@@ -1278,7 +1246,7 @@ DscIndices: Sources Release . .gz .bz2
             """\
 gnupghome {h}
 {m}
-""".format(h=os.path.join(mini_buildd.setup.HOME_DIR, ".gnupg"), m="morguedir +b/morguedir" if self.reprepro_morguedir else "")).save()
+""".format(h=os.path.join(mini_buildd.config.HOME_DIR, ".gnupg"), m="morguedir +b/morguedir" if self.reprepro_morguedir else "")).save()
 
         # (Re-)index
         self._mbd_reprepro().reindex()
@@ -1313,7 +1281,7 @@ gnupghome {h}
 
 
 def get_meta_distribution_map():
-    " Get a dict of the meta distributions: meta -> actual. "
+    """Get a dict of the meta distributions: meta -> actual."""
     result = {}
     for r in Repository.objects.all():
         for d in r.distributions.all():
@@ -1332,5 +1300,5 @@ def get_meta_distribution_map():
 
 
 def map_incoming_distribution(incoming_dist):
-    " Map incoming distribution to internal. "
+    """Map incoming distribution to internal."""
     return get_meta_distribution_map().get(incoming_dist, incoming_dist)
